#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : diags.py
# Contents: create detection diagrams
# Author  : Christian Borgelt
# History : 2014.01.07 file created
#-----------------------------------------------------------------------

from sys  import argv
from math import log10

tics = [2,'',4,'',6,'',8,'',10,'',12,'',14,'',16,'',18,'',20]
ddir = 'data/'

#-----------------------------------------------------------------------

def header (out, pro, addinit=''):
    out.write('verbatimtex\n')
    out.write('\\documentclass[12pt]{article}\n')
    out.write('\\usepackage{mathptmx}\n')
    out.write('\\def\\hnorm{\\hbox to0pt{\\phantom{g)}\\hss}}\n')
    out.write('\\begin{document}\n')
    out.write('etex\n\n')
    out.write('prologues := ' +str(pro) +';\n\n')
    out.write('numeric expand; expand = 1/3;\n\n')
    out.write('input axes;\n')
    out.write('input barchart;\n\n')
    out.write('vardef init_chart (expr xoff, yoff, zoff) =\n')
    out.write('  numeric data[][], n; n = 0;\n')
    out.write('  barchart.set_defaults("ptmr8r");\n')
    out.write('  barchart.set_view (-120, 36);\n')
    out.write('  barchart.set_light(-120, 75);\n')
    out.write('  barchart.clipcolor    := 0.7white;\n')
    out.write('  axes.title.size       := 1.2;  % 12pt\n')
    out.write('  axes.title.skip       := 1.2*fontsize "ptmr8r";\n')
    out.write('  axes.title.offset     := (0,2mm);\n')
    out.write('  axes.x.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.x.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.x.alabel.offset  := xoff;\n')
    out.write('  axes.y.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.y.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.y.alabel.offset  := yoff;\n')
    out.write('  axes.z.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.z.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.z.alabel.offset  := zoff;\n')
    out.write('  axes.z.grid.thickness := 0.4pt;\n')
    out.write('  axes.z.kinked         := true;\n')
    out.write(addinit)
    out.write('enddef;\n\n')
    out.write('vardef set_data (text list) =\n')
    out.write('  barchart.set_data(data[n], list);\n')
    out.write('  n := n+1;\n')
    out.write('enddef;\n\n')

#-----------------------------------------------------------------------

def footer (out):
    out.write('end.\n')

#-----------------------------------------------------------------------

def readfile (fname):
    with open(fname, 'r') as inp:
        c = [l.split() for l in inp]
    return dict([((int(x),int(y)),float(z)) for x,y,z in c])

#-----------------------------------------------------------------------

def barchart (out, fid, fname,
              exclude=None, dims=[40,40,24,0.84,0.08],
              title=('',''), toff=(0,0),
              xlabel='\\small pattern size $z$', xoff=(0,4.0),
              xvals=range(2,21), xtics=tics,
              ylabel='\\small coincidences $c$', yoff=(0,4.4),
              yvals=range(2,21), ytics=tics,
              zlabel='\\small log(\\#patterns)', zoff=(0,5.5),
              zscale='log',
              zrange=[-4,3], ztics=[-4,-3,-2,-1,0,1,2,3],
              znull=True, zcols=5):
    data = readfile(fname)
    excl = readfile(exclude) if exclude else set()
    name = fname[:fname.rfind('.txt')]
    out.write('beginfig(' +str(fid) +'); % '+name+'\n')
    out.write('  numeric zmin; zmin = %f;\n' % zrange[0])
    out.write('  numeric zmax; zmax = %f;\n' % zrange[1])
    out.write('  init_chart(('+str(xoff[0])+'mm, '
                              +str(xoff[1])+'mm), '
                          +'('+str(yoff[0])+'mm, '
                              +str(yoff[1])+'mm), '
                          +'('+str(zoff[0])+'mm, '
                              +str(zoff[1])+'mm));\n')
    out.write('  axes.title.offset    := ('+str(toff[0])+'mm, '
                                           +str(toff[1])+'mm);\n')
    out.write('  barchart.null.max    := zmin-0.5;\n')
    out.write('  barchart.invalid.max := zmin-1.5;\n')
    out.write('  barchart.invalid.barcolor := 0.5white;\n')
    out.write('  barchart.set_units('
                   +str(dims[0])+'mm/'+str(len(xvals))+', '
                   +str(dims[1])+'mm/'+str(len(yvals))+', '
                   +str(dims[2])+'mm/(zmax-zmin));\n')
    wd = str(dims[3])+','+str(dims[4])
    out.write('  barchart.set_ranges(('+str(len(xvals))+','+wd+'), '
                                   +'('+str(len(yvals))+','+wd+'), '
                                   +'(zmin,zmax));\n')
    out.write('  barchart.draw_xlabel(0, '
               +'btex '+xlabel+'\\hnorm etex);\n')
    out.write('  barchart.draw_xticks("'+str(xtics[0])+'"')
    for x in xtics[1:len(xvals)]: out.write(', "'+str(x)+'"')
    out.write(');\n')
    out.write('  barchart.draw_ylabel(0, '
               +'btex '+ylabel+'\\hnorm etex);\n')
    out.write('  barchart.draw_yticks("'+str(ytics[0])+'"')
    for y in ytics[1:len(yvals)]: out.write(', "'+str(y)+'"')
    out.write(');\n')
    out.write('  barchart.draw_zlabel(0, '
               +'btex '+zlabel+'\\hnorm etex);\n')
    out.write('  barchart.draw_zticks('+str(ztics[0]))
    for z in ztics[1:]: out.write(', '+str(z))
    out.write(');\n')
    out.write('  barchart.draw_title(-3, -1, '
                +'btex '+title[0]+'\\hnorm etex, "");\n')
    out.write('  barchart.draw_title(+3, -1, '
                +'btex '+title[1]+'\\hnorm etex, "");\n')
    out.write('  barchart.draw_frame;\n')
    for x in xvals:              # traverse the x-values
        out.write('  set_data(') # start the next data row
        first = True
        for y in yvals:          # traverse the y-values
            if not first: out.write(', ')
            else:         first = False
            z = data[(x,y)] if (x,y) in data else 0
            if zscale == 'log':
                z = log10(z) if z > 0 else zrange[0]-1
            if znull and z <= zrange[0]:
                z = zrange[0]-1
            if (x,y) in excl:
                z = zrange[0]-2
            out.write('%f' % (z if z <= zrange[1] else zrange[1]+1))
        out.write(');\n')       # print the values/fill data matrix
    out.write('  barchart.draw_data(data, 0, %d, expand);\n' % zcols)
    out.write('endfig;\n\n')    # draw the diagram

#-----------------------------------------------------------------------

if __name__ == '__main__':
    # attributes to describe chart properties:
    # fname:     name of the file containing the bar chart data
    #            each line contains a triplet consisting of
    #            z: the size/number of neurons
    #            c: the number of coincidences
    #            v: the value associated with the (z,c) pair
    #            no default, must be specified
    # exclude:   name of the file containing the excluded (z,c) pairs
    #            each line contains a triplet consisting of
    #            z: the pattern/assembly size  (number of neurons)
    #            c: the number of coincidences (pattern support)
    #            v: the value associated with the (z,c) pair (ignored)
    #            default: None
    # dims:      dimensions of the chart and the bars as a quintuple
    #            x: length of x-axis (in mm)
    #            y: length of y-axis (in mm)
    #            z: length of z-axis (in mm)
    #            w: width of bar relative to grid width
    #            o: offset of bar from left/bottom
    #            default: [40,40,24,0.84,0.08]
    # title:     text for chart title (two lines as strings)
    #            default: ('', '')
    # toff:      offset for title placement (in mm)
    #            default: (0,0)
    # xlabel:    label for x-axis (as LaTeX code)
    #            default: '\small pattern size $z$'
    # xoff:      offset for x-axis label placement (in mm)
    #            default: (0,4.2)
    # xvals:     list of x-axis values (as numbers)
    #            default: range(2,13)
    # xtics:     list of x-axis tic labels (as numbers or strings)
    #            default: [2,'',4,'',6,'',8,'',10,'',12,'',14,'',16]
    # ylabel:    label for y-axis (as LaTeX code)
    #            default: '\small coincidences $c$'
    # yoff:      offset for y-axis label placement (in mm)
    #            default: (0,4.2)
    # yvals:     list of y-axis values (as numbers)
    #            default: range(2,13)
    # ytics:     list of y-axis tic labels (as numbers or strings)
    #            default: [2,'',4,'',6,'',8,'',10,'',12,'',14,'',16]
    # zlabel:    label for z-axis (as LaTeX code)
    #            default: '\small rate'
    # zoff:      offset for y-axis label placement (in mm)
    #            default: (0,5.2)
    # zscale:    scale of vertical axis ('log' or 'linear')
    #            default: linear
    # zrange:    range of z-axis (as minimum and maximum value)
    #            default: [0,1]
    # ztics:     list of z-axis tic labels (as numbers)
    #            default: [0,0.2,0.4,0.6,0.8,1]
    # znull:     whether to show minimum z-value as a null bar
    #            default: True
    # zcols:     number of colors for color shading
    #            default: 5

    fid = 0
    tfs = '\\footnotesize '
    with open(argv[1] if len(argv) > 1 else 'diags.mp', 'w') as out:
        header(out, 2)
        for n in [100,80,60,40]:
            for p in [2,3,4,5]:
                for r in [30,20,10]:
                    for v in [3,2,1]:
                        for t in [3,2,1]:
                            for w in [3,4,5,6]:
                                pars = (n,p,r,t,v,w)
                                name = 'n%d_p%g_r%g_v%d_t%d_w%d' % pars
                                code = 'n%dp%dr%dv%dt%dw%d'      % pars
                                barchart(out, fid, ddir+'s_'+name+'.txt',
                                         title=(tfs+'surrogates',tfs+code))
                                fid += 1
                                barchart(out, fid, ddir+'e_'+name+'.txt',
                                         title=(tfs+'estimated', tfs+code))
                                fid += 1
        footer(out)
