#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : dotdisp.py
# Contents: generate dot displays of spike trains
# Author  : Christian Borgelt
# History : 2014.02.28 file created from spectrums.py
#-----------------------------------------------------------------------
from sys     import argv, stderr, path
from os.path import join
from random  import seed
from math    import ceil
NEURODIR = join('..', 'neuro')
if NEURODIR not in path: path.insert(0, NEURODIR)
from psts    import gentrain, genburst, warptrain, psts2mp

#-----------------------------------------------------------------------

def genpsts (ncnt=100, period=3, rate=20, nvar=1, tvar=1):
    '''genpsts (ncnt=100, period=3, rate=20, nvar=1, tvar=1)
Generate parallel spike trains.
ncnt    the number of neurons/spike trains
period  the length of the spike trains (in seconds)
rate    the average firing rate (average over neurons and time)
nvar    the factor for the variation over neurons (max to min)
tvar    the factor for the variation over time    (max to min)
returns a list of lists of spike times'''
    p = genburst(0.5,0.05)      # create two spike bursts
    p = [0.0 +10.0 for x in range(80)] \
      + [x*60+10.0 for x in p[0:50]]   \
      + [x*80+30.0 for x in p[50:500]] \
      + [x*70+30.0 for x in p[0:50]]   \
      + [x*50+10.0 for x in p[50:420]]
    x = []                      # concatenate to fill period
    for i in range(int(ceil(period))): x += p
    p = x[:min(len(x),int(ceil(period*1000)))]
    r = min(p)                  # compute the average rate and
    s = (tvar-1.0)*r/(max(p)-r) # the scaling factor for time variation
    p = [(x-r)*s+r for x in p]  # scale the rate profile
    s = period/sum(p)           # compute the scaling factor
    z = [s*p[0]]                # integrate the rate profile
    for i in range(1,len(p)): z.append(z[-1]+s*p[i])
    l = (nvar-1.0)/(nvar+1.0)   # compute the neuron variation range
    d = 2.0*l/(ncnt-1.0)        # and from it the rate step width
    l = 1.0-l                   # generate the spike trains
    return [warptrain(gentrain(period, rate*(l+n*d)), z)
            for n in range(ncnt)]

#-----------------------------------------------------------------------

def diags ():
    with open('trains.mp','w') as out:
        out.write('prologues    := 3;\n')
        out.write('defaultfont  := "ptmr8r";\n')
        out.write('defaultscale := 1;\n')
        out.write('labeloffset  := 0;\n')
        psts2mp(out, 0, genpsts(100,3,20,1,1), 3.0, width=57, height=20)
        psts2mp(out, 1, genpsts(100,3,20,3,1), 3.0, width=57, height=20)
        psts2mp(out, 2, genpsts(100,3,20,1,3), 3.0, width=57, height=20)
        psts2mp(out, 3, genpsts(100,3,20,3,3), 3.0, width=57, height=20)
        out.write('end.\n')

#-----------------------------------------------------------------------

if __name__ == '__main__':
    seed(1)
    diags()
