#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : spectra.py
# Contents: generate pattern spectra for various data sets
# Author  : Christian Borgelt
# History : 2014.01.07 file created
#           2014.02.19 variation of recording period added
#           2014.02.28 data generation separated from analysis
#           2014.05.27 modules placed in sibling directories
#           2014.11.18 adapted to modified genpsp() interface
#-----------------------------------------------------------------------
from sys     import argv, stderr, path
from os.path import join
from time    import strftime
from random  import seed
from math    import log, ceil
NEURODIR = join('..', 'neuro')
if NEURODIR not in path: path.insert(0, NEURODIR)
from psts    import gentrain, genburst, warptrain
try:
    from coco    import genpsp
except ImportError:
    FIMDIR   = join('..', 'fim')
    if FIMDIR   not in path: path.insert(0, FIMDIR)
    from patspec import genpsp
    print('Attention: Could not import from PyCoCo library!')
    print('           Using Python replacement (very slow)!')

#-----------------------------------------------------------------------

def genpsts (ncnt=100, period=3, rate=20, nvar=1, tvar=1):
    '''genpsts (ncnt=100, period=3, rate=20, nvar=1, tvar=1)
Generate parallel spike trains.
ncnt    the number of neurons/spike trains
period  the length of the spike trains (in seconds)
rate    the average firing rate (average over neurons and time)
nvar    the factor for the variation over neurons (max to min)
tvar    the factor for the variation over time    (max to min)
returns a list of lists of spike times'''
    p = genburst(0.5,0.05)      # create two spike bursts
    p = [0.0 +10.0 for x in range(80)] \
      + [x*60+10.0 for x in p[0:50]]   \
      + [x*80+30.0 for x in p[50:500]] \
      + [x*70+30.0 for x in p[0:50]]   \
      + [x*50+10.0 for x in p[50:420]]
    x = []                      # concatenate to fill period
    for i in range(int(ceil(period))): x += p
    p = x[:min(len(x),int(ceil(period*1000)))]
    r = min(p)                  # compute the average rate and
    s = (tvar-1.0)*r/(max(p)-r) # the scaling factor for time variation
    p = [(x-r)*s+r for x in p]  # scale the rate profile
    s = period/sum(p)           # compute the scaling factor
    z = [s*p[0]]                # integrate the rate profile
    for i in range(1,len(p)): z.append(z[-1]+s*p[i])
    l = (nvar-1.0)/(nvar+1.0)   # compute the neuron variation range
    d = 2.0*l/(ncnt-1.0)        # and from it the rate step width
    l = 1.0-l                   # generate the spike trains
    return [warptrain(gentrain(period, rate*(l+n*d)), z)
            for n in range(ncnt)]

#-----------------------------------------------------------------------

def makedata (n, p, r, v, t):
    name = 'n%d_p%g_r%g_v%d_t%d' % (n,p,r,v,t)
    psts = zip(range(1,n+1),genpsts(n,p,r,v,t))
    stderr.write(strftime('%Y.%m.%d %H:%M:%S '))
    stderr.write(name); stderr.write('\n')
    with open(join('data', name+'.txt'), 'w') as out:
        for i,t in psts:        # write the generated data set
            for s in t: out.write('%d %g\n' % (i,s))

#-----------------------------------------------------------------------

def makepsp (n, p, r, v, t, w):
    name = 'n%d_p%g_r%g_v%d_t%d' % (n,p,r,v,t)
    psts = dict()               # initialize the train dictionary
    with open(join('data', name+'.txt'), 'r') as inp:
        for line in inp:        # traverse the lines of the input file
            n,s = line.split()  # split into item and spike time
            s = float(s.strip())# collect points per item/neuron
            if n in psts: psts[n].append(s)
            else:         psts[n] = [s]
    psts = psts.items()         # convert to pairs (item, list of times)
    name += '_w%d' % w          # add the window width to the name
    stderr.write(strftime('%Y.%m.%d %H:%M:%S '))
    stderr.write(name); stderr.write('\n')

    # generate surrogate pattern spectrum 
    psp = genpsp(psts, target='s', width=0.001*w, supp=2, zmin=2,
                 report='#', algo='r', mode='', cnt=10000,
                 beg=0, end=p, surr='p', rand='r', sigma=0.003*w,
                 delta=0.01*w, seed=1)
    psp = sorted([(z,c,psp[z,c]) for z,c in psp])
    with open(join('data', 's_'+name+'.txt'), 'w') as out:
        for s in psp: out.write('%d %d %.16g\n' % s)

    # read surrogate pattern spectrum
    #psp = dict()               # initialize the dictionary
    #with open(join('data', 's_'+name+'.txt'), 'r') as inp:
    #    for line in inp:       # read pattern signatures and counts
    #        z,c,y = line.split()
    #        psp[int(z),int(c)] = float(y)
    return psp                  # return the pattern spectrum

#-----------------------------------------------------------------------

if __name__ == '__main__':
    seed(1)                     # seed random number generator
    if len(argv) > 1:           # if an argument is given,
        stderr.write('generating data sets ...\n')
        for n in [100,80,60,40]:# number of neurons
            for p in [5,4,3,2]: # length of trains (duration)
                for r in [30,20,10]:      # average firing rate
                    for v in [3,2,1]:     # variation over neurons
                        for t in [3,2,1]: # and time
                            makedata(n, p, r, v, t)
        stderr.write('done.\n')
        exit()
    stderr.write('generating pattern spectra ...\n')
    for n in [100,80,60,40]:    # number of neurons
        for p in [5,4,3,2]:     # length of trains (duration)
            for r in [30,20,10]:      # average firing rate
                for v in [3,2,1]:     # variation over neurons
                    for t in [3,2,1]: # and time
                        for w in [6,5,4,3]: # analysis window width (ms)
                            makepsp(n, p, r, v, t, w)
    stderr.write('done.\n')
